==> acquia_dam-1.0.0.tar.gz: acquia_dam/src/Plugin/media/Source/Asset.php <==
<?php

declare(strict_types=1);

namespace Drupal\acquia_dam\Plugin\media\Source;

use Drupal\acquia_dam\Entity\MediaEmbedsField;
use Drupal\acquia_dam\Entity\MediaSourceField;
use Drupal\Component\Render\PlainTextOutput;
use Drupal\Core\File\Exception\FileException;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\media\MediaInterface;
use Drupal\media\MediaSourceBase;
use Drupal\media\MediaTypeInterface;
use GuzzleHttp\Exception\TransferException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Media source for DAM assets.
 *
 * @MediaSource(
 *   id = "acquia_dam_asset",
 *   label = @Translation("Asset"),
 *   description = @Translation("Use an asset from the Acquia DAM"),
 *   allowed_field_types = {"acquia_dam_asset"},
 *   default_thumbnail_filename = "no-thumbnail.png",
 *   deriver = "Drupal\acquia_dam\Plugin\media\Source\AssetDeriver",
 *   asset_search_key = "",
 *   asset_search_value = "",
 * )
 */
final class Asset extends MediaSourceBase {

  /**
   * The DAM client factory.
   *
   * @var \Drupal\acquia_dam\Client\AcquiaDamClientFactory
   */
  private $clientFactory;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  private $httpClient;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  private $fileSystem;

  /**
   * The token replacement service.
   *
   * @var \Drupal\Core\Utility\Token
   */
  private $token;

  /**
   * Logger channel interface.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  private $damLoggerChannel;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->clientFactory = $container->get('acquia_dam.client.factory');
    $instance->httpClient = $container->get('http_client');
    $instance->fileSystem = $container->get('file_system');
    $instance->token = $container->get('token');
    $instance->damLoggerChannel = $container->get('logger.channel.acquia_dam');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadataAttributes(): array {
    return [
      'id' => $this->t('Asset ID'),
      'version_id' => $this->t('Version ID'),
      'external_id' => $this->t('External ID'),
      'created_date' => $this->t('Created date'),
      'filename' => $this->t('File name'),
      'size' => $this->t('Size'),
      'thumbnail_uri' => $this->t('Thumbnail local URI'),
      'embeds' => $this->t('Embed Data'),
    ];
  }

  /**
   * {@inheritdoc}
   *
   * Disable PHPMD.CyclomaticComplexity due to the switch statement, which is
   * a pattern used in all of the implementations of this method.
   *
   * @SuppressWarnings(PHPMD.CyclomaticComplexity)
   */
  public function getMetadata(MediaInterface $media, $attribute_name) {
    $asset_id = $this->getSourceFieldValue($media)['asset_id'];

    if (empty($asset_id)) {
      return NULL;
    }

    try {
      $asset = $this->clientFactory->getSiteClient()->getAsset($asset_id);
    }
    catch (\Exception $exception) {
      $this->damLoggerChannel->error(sprintf(
          'Following error occurred while trying to get asset from dam. Asset: %s, error: %s',
          $asset_id,
          $exception->getMessage()
        )
      );
      return NULL;
    }

    if ($asset === NULL) {
      // @todo report error?
      if ($attribute_name === 'default_name') {
        return parent::getMetadata($media, $attribute_name);
      }
      return NULL;
    }

    // @todo flush these out.
    switch ($attribute_name) {
      case 'filename':
      case 'default_name':
        return $asset['filename'];

      case 'id':
      case 'version_id':
        return $asset['id'];

      case 'created_date':
        return $asset['created_date'];

      case 'size':
        return $asset['file_properties']['size_in_kbytes'] * 1024;

      case 'thumbnail_uri':
        return $this->getLocalThumbnailUri($asset);

      case 'embeds':
        // Check if the field is empty or not if so pull it from the API.
        return $this->getEmbeds($media) ?? $asset['embeds'];

      case 'thumbnails':
        return $asset['thumbnails'];

      case 'image_properties':
        return $asset['file_properties']['image_properties'];

      case 'video_properties':
        return $asset['file_properties']['video_properties'];

      default:
        break;
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['source_field']['#default_value'] = MediaSourceField::SOURCE_FIELD_NAME;
    $form['source_field']['#disabled'] = TRUE;
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceFieldValue(MediaInterface $media): array {
    $items = $media->get($this->getSourceFieldName());
    if ($items->isEmpty()) {
      return [
        'asset_id' => '',
        'version_id' => '',
      ];
    }
    return $items->first()->getValue();
  }

  /**
   * {@inheritdoc}
   */
  public function createSourceField(MediaTypeInterface $type) {
    return $this->getSourceFieldDefinition($type);
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceFieldDefinition(MediaTypeInterface $type) {
    return MediaSourceField::getFieldDefinition('media', $type->id(), $type->label());
  }

  /**
   * {@inheritdoc}
   */
  protected function getSourceFieldName() {
    return MediaSourceField::SOURCE_FIELD_NAME;
  }

  /**
   * Returns the local URI for a resource thumbnail.
   *
   * @param array $asset
   *   The asset.
   *
   * @return string
   *   The local thumbnail URI, or NULL if it could not be downloaded.
   *
   * @see \Drupal\media\Plugin\media\Source\OEmbed::getLocalThumbnailUri
   */
  protected function getLocalThumbnailUri(array $asset): string {
    $directory = 'public://acquia_dam_thumbnails/[date:custom:Y-m]';
    $directory = $this->token->replace($directory);
    $directory = PlainTextOutput::renderFromHtml($directory);
    if (!$this->fileSystem->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS)) {
      return '';
    }

    $thumbnail_url = $asset['thumbnails']['160px']['url'];
    try {
      $response = $this->httpClient->request('GET', $thumbnail_url);
      if ($response->getStatusCode() === 200) {
        $content_type = $response->getHeaderLine('Content-Type');

        $filename = pathinfo($asset['filename'], PATHINFO_FILENAME);
        $extension = '';
        // @todo ?? There's no mimetype to extension mapping?
        //   \Drupal\Core\File\MimeType\ExtensionMimeTypeGuesser has mapping but
        //   takes an extension to get mimetype, not the reverse mapping.
        if ($content_type === 'image/png') {
          $extension = '.png';
        }
        elseif ($content_type === 'image/jpeg') {
          $extension = '.jpg';
        }

        $local_thumbnail_uri = $directory . DIRECTORY_SEPARATOR . $filename . $extension;
        $this->fileSystem->saveData((string) $response->getBody(), $local_thumbnail_uri, FileSystemInterface::EXISTS_REPLACE);
        return $local_thumbnail_uri;
      }
    }
    catch (TransferException $e) {
      return '';
    }
    catch (FileException $e) {
      return '';
    }
    return '';
  }

  /**
   * Helper function that check if the media embeds field as value.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media object.
   *
   * @return null|string
   *   Returns the embed value if present.
   */
  protected function getEmbeds(MediaInterface $media) {
    if ($media->get(MediaEmbedsField::EMBED_FIELD_NAME)->isEmpty()) {
      return NULL;
    }
    else {
      return $media->get(MediaEmbedsField::EMBED_FIELD_NAME)->value;
    }
  }

}

==> acquia_dam-1.0.1.tar.gz: acquia_dam/src/Plugin/media/Source/Asset.php <==
<?php

declare(strict_types=1);

namespace Drupal\acquia_dam\Plugin\media\Source;

use Drupal\acquia_dam\Entity\MediaEmbedsField;
use Drupal\acquia_dam\Entity\MediaSourceField;
use Drupal\acquia_dam\Plugin\Field\FieldType\AssetItem;
use Drupal\Core\File\Exception\FileException;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\media\MediaInterface;
use Drupal\media\MediaSourceBase;
use Drupal\media\MediaTypeInterface;
use GuzzleHttp\Exception\TransferException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Media source for DAM assets.
 *
 * @MediaSource(
 *   id = "acquia_dam_asset",
 *   label = @Translation("Asset"),
 *   description = @Translation("Use an asset from the Acquia DAM"),
 *   allowed_field_types = {"acquia_dam_asset"},
 *   default_thumbnail_filename = "no-thumbnail.png",
 *   deriver = "Drupal\acquia_dam\Plugin\media\Source\AssetDeriver",
 *   asset_search_key = "",
 *   asset_search_value = "",
 * )
 */
final class Asset extends MediaSourceBase {

  /**
   * The DAM client factory.
   *
   * @var \Drupal\acquia_dam\Client\AcquiaDamClientFactory
   */
  private $clientFactory;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  private $httpClient;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  private $fileSystem;

  /**
   * The token replacement service.
   *
   * @var \Drupal\Core\Utility\Token
   */
  private $token;

  /**
   * Logger channel interface.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  private $damLoggerChannel;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->clientFactory = $container->get('acquia_dam.client.factory');
    $instance->httpClient = $container->get('http_client');
    $instance->fileSystem = $container->get('file_system');
    $instance->token = $container->get('token');
    $instance->damLoggerChannel = $container->get('logger.channel.acquia_dam');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getMetadataAttributes(): array {
    $attributes = [
      'created_date' => $this->t('Created date'),
      'filename' => $this->t('File name'),
      'size' => $this->t('Size'),
    ];
    if ($this->configFactory->get('acquia_dam.settings')->get('allowed_metadata')) {
      return array_merge($attributes, $this->configFactory->get('acquia_dam.settings')->get('allowed_metadata'));
    }
    return $attributes;
  }

  /**
   * {@inheritdoc}
   *
   * Disable PHPMD.CyclomaticComplexity due to the switch statement, which is
   * a pattern used in all the implementations of this method.
   *
   * @SuppressWarnings(PHPMD.CyclomaticComplexity)
   */
  public function getMetadata(MediaInterface $media, $attribute_name) {
    [$asset_id, $version_id] = array_values($this->getSourceFieldValue($media));

    if (empty($asset_id)) {
      return NULL;
    }
    if ($version_id === NULL) {
      $version_id = '';
    }

    try {
      $asset = $this->clientFactory->getSiteClient()->getAsset($asset_id, $version_id);
    }
    catch (\Exception $exception) {
      $this->damLoggerChannel->error(sprintf(
          'Following error occurred while trying to get asset from dam. Asset: %s, error: %s',
          $asset_id,
          $exception->getMessage()
        )
      );
      return NULL;
    }

    if ($asset === NULL) {
      // @todo report error?
      if ($attribute_name === 'default_name') {
        return parent::getMetadata($media, $attribute_name);
      }
      return NULL;
    }

    // @todo flush these out.
    switch ($attribute_name) {
      case 'filename':
      case 'default_name':
        return $asset['filename'];

      case 'id':
      case 'version_id':
        return $asset['id'];

      case 'created_date':
        return $asset['created_date'];

      case 'size':
        return $asset['file_properties']['size_in_kbytes'] * 1024;

      case 'thumbnail_uri':
        return $this->getLocalThumbnailUri($asset_id, $version_id, $asset);

      case 'embeds':
        // Check if the field is empty or not if so pull it from the API.
        return $this->getEmbeds($media) ?? $asset['embeds'];

      case 'thumbnails':
        return $asset['thumbnails'];

      case 'image_properties':
        return $asset['file_properties']['image_properties'];

      case 'video_properties':
        return $asset['file_properties']['video_properties'];

      default:
        // DAM contains multiple metadata type so it requires this check.
        if (array_key_exists($attribute_name, $asset['metadata']['fields'])) {
          return implode(',', $asset['metadata']['fields'][$attribute_name]);
        }
        return NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form['source_field']['#default_value'] = MediaSourceField::SOURCE_FIELD_NAME;
    $form['source_field']['#disabled'] = TRUE;
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceFieldValue(MediaInterface $media): array {
    $items = $media->get($this->getSourceFieldName());
    if ($items->isEmpty()) {
      return [
        'asset_id' => '',
        'version_id' => '',
      ];
    }
    $field_item = $items->first();
    assert($field_item instanceof AssetItem);
    // The ::getValue method on FieldItem only returns an array where properties
    // have been initiated with a value. It does not return properties that have
    // no value. Using ::toArray ensures the result has `version_id`, which may
    // be empty when a media item is first saved.
    return $field_item->toArray();
  }

  /**
   * {@inheritdoc}
   */
  public function createSourceField(MediaTypeInterface $type) {
    return $this->getSourceFieldDefinition($type);
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceFieldDefinition(MediaTypeInterface $type) {
    return MediaSourceField::getFieldDefinition('media', $type->id(), $type->label());
  }

  /**
   * {@inheritdoc}
   */
  protected function getSourceFieldName() {
    return MediaSourceField::SOURCE_FIELD_NAME;
  }

  /**
   * Returns the local URI for a resource thumbnail.
   *
   * @param string $asset_id
   *   The asset ID.
   * @param string $version_id
   *   The version ID, an empty string is allowed if version unknown.
   * @param array $asset
   *   The asset.
   *
   * @return string
   *   The local thumbnail URI, or NULL if it could not be downloaded.
   *
   * @see \Drupal\media\Plugin\media\Source\OEmbed::getLocalThumbnailUri
   */
  protected function getLocalThumbnailUri(string $asset_id, string $version_id, array $asset): string {
    if ($version_id === '') {
      // The version ID was not yet set. This happens when the media is saved
      // without a version ID and Media::prepareSave is invoked before preSave
      // where the version is populated. Give a generic version ID for the
      // filename of this thumbnail.
      $version_id = 'default';
    }
    $directory = 'public://acquia_dam_thumbnails/' . $asset_id;
    if (!$this->fileSystem->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS)) {
      return '';
    }
    // If there is an existing local thumbnail for the version, return it.
    $files = $this->fileSystem->scanDirectory($directory, "/^$version_id\..*/");
    if (count($files) > 0) {
      return reset($files)->uri;
    }

    $thumbnail_url = $asset['thumbnails']['160px']['url'];
    try {
      $response = $this->httpClient->request('GET', $thumbnail_url);
      if ($response->getStatusCode() === 200) {
        $filename = $asset['filename'];
        if ($response->hasHeader('Content-Disposition')) {
          $disposition = $response->getHeader('Content-Disposition')[0];
          preg_match('/filename="(.*)"/', $disposition, $matches);
          if (count($matches) > 1) {
            $filename = $matches[1];
          }
        }

        $extension = pathinfo($filename, PATHINFO_EXTENSION);

        $local_thumbnail_uri = $directory . DIRECTORY_SEPARATOR . "$version_id.$extension";
        $this->fileSystem->saveData((string) $response->getBody(), $local_thumbnail_uri, FileSystemInterface::EXISTS_REPLACE);
        return $local_thumbnail_uri;
      }
    }
    catch (TransferException $e) {
      $this->damLoggerChannel->error(sprintf(
          'Unable to download thumbnail at %s: %s %s',
          $thumbnail_url,
          get_class($e),
          $e->getMessage()
        ));
      return '';
    }
    catch (FileException $e) {
      $this->damLoggerChannel->error(sprintf(
        'Unable to download thumbnail at %s: %s %s',
        $thumbnail_url,
        get_class($e),
        $e->getMessage()
      ));
      return '';
    }
    return '';
  }

  /**
   * Helper function that check if the media embeds field as value.
   *
   * @param \Drupal\media\MediaInterface $media
   *   The media object.
   *
   * @return null|string
   *   Returns the embed value if present.
   */
  protected function getEmbeds(MediaInterface $media) {
    if ($media->get(MediaEmbedsField::EMBED_FIELD_NAME)->isEmpty()) {
      return NULL;
    }

    return $media->get(MediaEmbedsField::EMBED_FIELD_NAME)->value;
  }

}
